@@ -1,4 +1,4 @@
-#' @title Create table from backtest summary.
+#' @title Create table from backtest summary
 #' 
 #' @description After performing a backtest with \code{\link{portfolioBacktest}} 
 #' and obtaining a summary of the performance measures with 
@@ -14,12 +14,14 @@
 #' @param type Type of table. Valid options: \code{"simple", "DT", "grid.table"}. Default is 
 #'             \code{"simple"} and generates a simple matrix (with the other choices the 
 #'             corresponding package must be installed).
-#' @param order_col Column number of the performance measure to be used to sort the rows 
-#'                  (only used for table \code{type = "DT"}). By default the last column 
-#'                  will be used.
+#' @param order_col Column number or column name of the performance measure to be used to 
+#'                  sort the rows (only used for table \code{type = "DT"}). By default the 
+#'                  last column will be used.
 #' @param order_dir Direction to be used to sort the rows (only used for table 
 #'                  \code{type = "DT"}). Valid options: \code{"asc", "desc"}. 
 #'                  Default is \code{"asc"}.
+#' @param page_length Page length for the table (only used for table \code{type = "DT"}). 
+#'                    Default is \code{10}.
 #' 
 #' @author Daniel P. Palomar and Rui Zhou
 #' 
@@ -41,7 +43,8 @@
 #' }
 #' 
 #' # do backtest
-#' bt <- portfolioBacktest(list("Quintile" = quintile_portfolio), dataset10,
+#' bt <- portfolioBacktest(list("Quintile" = quintile_portfolio), 
+#'                         dataset10,
 #'                         benchmark = c("uniform", "index"))
 #' 
 #' # now we can obtain the table
@@ -52,7 +55,7 @@
 #' 
 #' @export
 summaryTable <- function(bt_summary, measures = NULL, type = c("simple", "DT", "grid.table"), 
-                         order_col = NULL, order_dir = c("asc", "desc")) {
+                         order_col = NULL, order_dir = c("asc", "desc"), page_length = 10) {
   if (is.null(measures)) measures <- c("cpu time", rownames(bt_summary$performance_summary))  # by default use all
   # extract performance measures
   real_measures <- intersect(measures, rownames(bt_summary$performance_summary))
@@ -67,9 +70,12 @@ summaryTable <- function(bt_summary, measures = NULL, type = c("simple", "DT", "
          "DT" = {
            if (!requireNamespace("DT", quietly = TRUE)) 
              stop("Please install package \"DT\" or choose another table type", call. = FALSE)
-           if (is.null(order_col)) order_col <- ncol(performance)
+           if (is.character(order_col)) order_col <- which(colnames(performance) == order_col)
+           if (is.null(order_col) || length(order_col) == 0) order_col <- ncol(performance)
            order_dir <- match.arg(order_dir)
-           p <- DT::datatable(performance, options = list(dom = 't', pageLength = 15, scrollX = TRUE, order = list(order_col, order_dir)))
+           p <- DT::datatable(performance, 
+                              options = list(pageLength = page_length, scrollX = TRUE, order = list(order_col, order_dir)),
+                              caption = "Leaderboard:")
            p <- DT::formatStyle(p, 0, target = "row", fontWeight = DT::styleEqual(c("uniform", "index"), c("bold", "bold")))
            if ("annual volatility" %in% colnames(performance))
              p <- DT::formatPercentage(p, "annual volatility", 1)
@@ -87,7 +93,7 @@ summaryTable <- function(bt_summary, measures = NULL, type = c("simple", "DT", "
 
 
 
-#' @title Create barplot from backtest summary.
+#' @title Create barplot from backtest summary
 #' 
 #' @description After performing a backtest with \code{\link{portfolioBacktest}} 
 #' and obtaining a summary of the performance measures with 
@@ -97,7 +103,7 @@ summaryTable <- function(bt_summary, measures = NULL, type = c("simple", "DT", "
 #' 
 #' @inheritParams summaryTable
 #' @param type Type of plot. Valid options: \code{"ggplot2", "simple"}. Default is 
-#'             \code{"ggplot2"} (the package \code{ggplot2} must be installed).
+#'             \code{"ggplot2"}.
 #' @param ... Additional parameters (only used for plot \code{type = "simple"}); 
 #'            for example: \code{mar} for margins as in \code{par()},
 #'                         \code{inset} for the legend inset as in \code{legend()},
@@ -135,6 +141,7 @@ summaryTable <- function(bt_summary, measures = NULL, type = c("simple", "DT", "
 #' 
 #' @importFrom grDevices topo.colors
 #' @importFrom graphics barplot legend par
+#' @import ggplot2
 #' @export
 summaryBarPlot <- function(bt_summary, measures = NULL, type = c("ggplot2", "simple"), ...) {
   # extract table
@@ -161,22 +168,19 @@ summaryBarPlot <- function(bt_summary, measures = NULL, type = c("ggplot2", "sim
            par(old_par)
          },
          "ggplot2" = {
-           if (!requireNamespace("ggplot2", quietly = TRUE)) 
-             stop("Please install package \"ggplot2\" or choose another plot type", call. = FALSE)
            df <- as.data.frame.table(res_table)
-           Var1 <- Freq <- NULL  # ugly hack to deal with CRAN note
-           ggplot2::ggplot(df, ggplot2::aes(x = Var1, y = Freq, fill = Var1)) + 
-             ggplot2::geom_bar(stat = "identity") +  #position = position_dodge()
-             ggplot2::scale_x_discrete(breaks = NULL) +
-             ggplot2::facet_wrap(~ Var2, scales = "free_y") +
-             ggplot2::labs(title = params$main, x = NULL, y = NULL, fill = NULL)
+           ggplot(df, aes_string(x = "Var1", y = "Freq", fill = "Var1")) + 
+             geom_bar(stat = "identity") +  #position = position_dodge()
+             scale_x_discrete(breaks = NULL) +
+             facet_wrap(~ Var2, scales = "free_y") +
+             labs(title = params$main, x = NULL, y = NULL, fill = NULL)
          },
          stop("Barplot type unknown."))
 }
 
 
 
-#' @title Create boxplot from backtest results.
+#' @title Create boxplot from backtest results
 #' 
 #' @description Create boxplot from a portfolio backtest obtained with the function 
 #' \code{\link{portfolioBacktest}}. By default the boxplot is based on the 
@@ -189,7 +193,7 @@ summaryBarPlot <- function(bt_summary, measures = NULL, type = c("ggplot2", "sim
 #'                 \code{"Sterling ratio"}, \code{"Omega ratio"}, and \code{"ROT bps"}.
 #'                  Default is \code{"Sharpe ratio"}.
 #' @param type Type of plot. Valid options: \code{"ggplot2", "simple"}. Default is 
-#'             \code{"ggplot2"} (the package \code{ggplot2} must be installed).
+#'             \code{"ggplot2"}.
 #' @param ... Additional parameters. For example: 
 #'            \code{mar} for margins as in \code{par()} (for the case of plot \code{type = "simple"}); and
 #'            \code{alpha} for the alpha of each backtest dot (for the case of plot \code{type = "ggplot2"}), 
@@ -221,8 +225,10 @@ summaryBarPlot <- function(bt_summary, measures = NULL, type = c("ggplot2", "sim
 #' backtestBoxPlot(bt, "Sharpe ratio", type = "simple")
 #' }
 #' 
+#' @importFrom grDevices topo.colors
 #' @importFrom graphics boxplot par
 #' @importFrom stats quantile
+#' @import ggplot2
 #' @export
 backtestBoxPlot <- function(bt, measure = "Sharpe ratio", type = c("ggplot2", "simple"), ...) {
   # extract correct performance measure
@@ -248,8 +254,6 @@ backtestBoxPlot <- function(bt, measure = "Sharpe ratio", type = c("ggplot2", "s
            par(old_par)
          },
          "ggplot2" = {
-           if (!requireNamespace("ggplot2", quietly = TRUE)) 
-             stop("Please install package \"ggplot2\" or choose another plot type", call. = FALSE)
            if (is.null(params$alpha)) params$alpha <- 0.4  # this is for the points (set to 0 if not want them)
            limits <- apply(res_table, 2, function(x) {
              lquartile <- quantile(x, 0.25, na.rm = TRUE)
@@ -259,13 +263,12 @@ backtestBoxPlot <- function(bt, measure = "Sharpe ratio", type = c("ggplot2", "s
            })
            plot_limits <- c(min(limits["limit_min", ]), max(limits["limit_max", ]))
            df <- as.data.frame.table(res_table)
-           Var2 <- Freq <- NULL  # ugly hack to deal with CRAN note
-           ggplot2::ggplot(df, ggplot2::aes(x = Var2, y = Freq, fill = Var2)) +
-             ggplot2::geom_boxplot(show.legend = FALSE) +  # (outlier.shape = NA)
-             ggplot2::geom_point(size = 0.5, alpha = params$alpha, show.legend = FALSE) +  # geom_jitter(width = 0) +
-             ggplot2::scale_x_discrete(limits = rev(levels(df$Var2))) +
-             ggplot2::coord_flip(ylim = plot_limits) + 
-             ggplot2::labs(title = measure, x = NULL, y = NULL)
+           ggplot(df, aes_string(x = "Var2", y = "Freq", fill = "Var2")) +
+             geom_boxplot(show.legend = FALSE) +  # (outlier.shape = NA)
+             geom_point(size = 0.5, alpha = params$alpha, show.legend = FALSE) +  # geom_jitter(width = 0) +
+             scale_x_discrete(limits = rev(levels(df$Var2))) +
+             coord_flip(ylim = plot_limits) + 
+             labs(title = measure, x = NULL, y = NULL)
          },
          stop("Boxplot type unknown."))
 }
