@@ -1,4 +1,4 @@
-#' @title Backtest multiple portfolios over multiple datasets of stock prices in a rolling-window basis.
+#' @title Backtest multiple portfolios over multiple datasets of stock prices in a rolling-window basis
 #'
 #' @description Automated backtesting of multiple portfolios over multiple 
 #' datasets of stock prices in a rolling-window fashion. 
@@ -45,10 +45,26 @@
 #' @param T_rolling_window Length of the lookback rolling window (default is \code{252}).
 #' @param optimize_every How often the portfolio is to be optimized (default is \code{20}).
 #' @param rebalance_every How often the portfolio is to be rebalanced (default is \code{1}).
+#' @param execution String that can be either \code{"same day"} (default) or \code{"next day"}. 
+#'                  At the rebalancing period \code{t}, the portfolio has used information up to (and including)
+#'                  period \code{t}. Same day execution means one can get into the position at that period \code{t},
+#'                  whereas the next day execution means that one can only get into the position the following day.
+#' @param cost List containing four different types of transaction costs (common for all assets) 
+#'             for buying, selling, shorting, and long leveraging. The default is 
+#'             \code{cost = list(buy = 0e-4, sell = 0e-4, short = 0e-4, long_leverage = 0e-4)}.
+#'             If some elements are not specified then they will be automatically set to zero.
 #' @param cpu_time_limit Time limit for executing each portfolio function over a single data set 
 #'                       (default is \code{Inf}, so no time limit).
-#' @param return_portfolio Logical value indicating whether to return portfolios (default is \code{TRUE}).
+#' @param return_portfolio Logical value indicating whether to return the portfolios (default is \code{TRUE}).
+#'                         Two portfolios are returned: \code{w_designed} is the designed portfolio at each
+#'                         given rebalancing period (using all the information up to and including that period,
+#'                         which can be executed either on the same day or the following day)
+#'                         and \code{w_bop} is the "beginning-of-period" portfolio (i.e., at each period it contains 
+#'                         the weights held in the market in the previous period so that the portfolio return at 
+#'                         that period is just the product of the asset returns and \code{w_bop} at that period.)
 #' @param return_returns Logical value indicating whether to return the portfolio returns (default is \code{TRUE}).
+#'                       Two series are returned: \code{return} with the portfolio returns and \code{wealth}
+#'                       with the portfolio wealth (aka cumulative P&L).
 #' 
 #' @return List with the portfolio backtest results, see 
 #'         \href{https://CRAN.R-project.org/package=portfolioBacktest/vignettes/PortfolioBacktest.html#result-format}{vignette-result-format}
@@ -94,7 +110,9 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
                               paral_portfolios = 1, paral_datasets = 1,
                               show_progress_bar = FALSE, benchmark = NULL, 
                               shortselling = TRUE, leverage = Inf,
-                              T_rolling_window = 252, optimize_every = 20, rebalance_every = 1,
+                              T_rolling_window = 252, optimize_every = 20, rebalance_every = 1, 
+                              execution = c("same day", "next day"), 
+                              cost = list(buy = 0e-4, sell = 0e-4, short = 0e-4, long_leverage = 0e-4),
                               cpu_time_limit = Inf,
                               return_portfolio = TRUE, return_returns = TRUE) {
   ####### error control ########
@@ -103,6 +121,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
   if (paral_portfolios < 1 || paral_datasets < 1) stop("Parallel number must be a positive interger.")
   if (is.null(folder_path) && is.null(portfolio_funs)) stop("The \"folder_path\" and \"portfolio_fun_list\" cannot be both NULL.")
   if (!is.null(portfolio_funs) && !is.list(portfolio_funs)) portfolio_funs <- list(portfolio_funs)
+  cost <- modifyList(list(buy = 0, sell = 0, short = 0, long_leverage = 0), cost)
+  if (length(cost) != 4) stop("Problem in specifying the cost: the elements can only be buy, sell, short, and long_leverage.")
   ##############################
   
   # when portfolio_funs is passed
@@ -117,14 +137,16 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
     safeEvalPortf <- function(portfolio_fun, dataset_list, price_name,
                               paral_datasets, show_progress_bar,
                               shortselling, leverage,
-                              T_rolling_window, optimize_every, rebalance_every,
+                              T_rolling_window, optimize_every, rebalance_every, 
+                              execution, cost,
                               cpu_time_limit,
                               return_portfolio, return_returns) {
       packages_default <- search()  # snap the default packages
       res <- singlePortfolioBacktest(portfolio_fun, dataset_list, price_name, market = FALSE,
                                      paral_datasets, show_progress_bar,
                                      shortselling, leverage,
-                                     T_rolling_window, optimize_every, rebalance_every,
+                                     T_rolling_window, optimize_every, rebalance_every, 
+                                     execution, cost,
                                      cpu_time_limit,
                                      return_portfolio, return_returns)
       packages_now <- search()  # detach the newly loaded packages
@@ -141,7 +163,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
         result[[i]] <- safeEvalPortf(portfolio_funs[[i]], dataset_list, price_name,
                                      paral_datasets, show_progress_bar,
                                      shortselling, leverage,
-                                     T_rolling_window, optimize_every, rebalance_every,
+                                     T_rolling_window, optimize_every, rebalance_every, 
+                                     execution, cost,
                                      cpu_time_limit,
                                      return_portfolio, return_returns)
       }
@@ -166,7 +189,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
         return(list(safeEvalPortf(portfolio_fun, dataset_list, price_name,
                                   paral_datasets, show_progress_bar,
                                   shortselling, leverage,
-                                  T_rolling_window, optimize_every, rebalance_every,
+                                  T_rolling_window, optimize_every, rebalance_every, 
+                                  execution, cost,
                                   cpu_time_limit,
                                   return_portfolio, return_returns)))
       }
@@ -184,7 +208,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
     safeEvalFolder <- function(folder_path, file, dataset_list__, price_name,
                                paral_datasets, show_progress_bar,
                                shortselling, leverage,
-                               T_rolling_window, optimize_every, rebalance_every,
+                               T_rolling_window, optimize_every, rebalance_every, 
+                               execution, cost,
                                cpu_time_limit,
                                return_portfolio, return_returns) {
       packages_default <- search()  # snap the default packages
@@ -194,7 +219,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
         res <- singlePortfolioBacktest(portfolio_fun, dataset_list__, price_name, market = FALSE,
                                        paral_datasets, show_progress_bar,
                                        shortselling, leverage,
-                                       T_rolling_window, optimize_every, rebalance_every,
+                                       T_rolling_window, optimize_every, rebalance_every, 
+                                       execution, cost,
                                        cpu_time_limit,
                                        return_portfolio, return_returns)
         NULL
@@ -216,7 +242,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
         result[[i]] <- safeEvalFolder(folder_path, files[i], dataset_list__, price_name,
                                       paral_datasets, show_progress_bar,
                                       shortselling, leverage,
-                                      T_rolling_window, optimize_every, rebalance_every,
+                                      T_rolling_window, optimize_every, rebalance_every, 
+                                      execution, cost,
                                       cpu_time_limit,
                                       return_portfolio, return_returns)
       }
@@ -238,7 +265,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
         return(list(safeEvalFolder(folder_path, file, dataset_list__, price_name,
                                    paral_datasets, show_progress_bar,
                                    shortselling, leverage,
-                                   T_rolling_window, optimize_every, rebalance_every,
+                                   T_rolling_window, optimize_every, rebalance_every, 
+                                   execution, cost,
                                    cpu_time_limit,
                                    return_portfolio, return_returns)))
       }
@@ -253,7 +281,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
   res_benchmark <- benchmarkBacktest(dataset_list, benchmark, price_name,
                                      paral_datasets, show_progress_bar,
                                      shortselling, leverage,
-                                     T_rolling_window, optimize_every, rebalance_every,
+                                     T_rolling_window, optimize_every, rebalance_every, 
+                                     execution, cost,
                                      cpu_time_limit,
                                      return_portfolio, return_returns)
   result <- c(result, res_benchmark)
@@ -269,7 +298,8 @@ portfolioBacktest <- function(portfolio_funs = NULL, dataset_list, folder_path =
 benchmarkBacktest <- function(dataset_list, benchmark, price_name,
                               paral_datasets, show_progress_bar,
                               shortselling, leverage,
-                              T_rolling_window, optimize_every, rebalance_every,
+                              T_rolling_window, optimize_every, rebalance_every, 
+                              execution, cost,
                               cpu_time_limit,
                               return_portfolio, return_returns) {
   res <- list()
@@ -278,7 +308,8 @@ benchmarkBacktest <- function(dataset_list, benchmark, price_name,
     res$uniform <- singlePortfolioBacktest(uniform_portfolio_fun, dataset_list, price_name, market = FALSE,
                                            paral_datasets, show_progress_bar,
                                            shortselling, leverage,
-                                           T_rolling_window, optimize_every, rebalance_every,
+                                           T_rolling_window, optimize_every, rebalance_every, 
+                                           execution, cost,
                                            cpu_time_limit,
                                            return_portfolio, return_returns)
   }
@@ -287,7 +318,8 @@ benchmarkBacktest <- function(dataset_list, benchmark, price_name,
     res$index <- singlePortfolioBacktest(portfolio_fun = NULL, dataset_list, price_name, market = TRUE,
                                          paral_datasets, show_progress_bar,
                                          shortselling, leverage,
-                                         T_rolling_window, optimize_every, rebalance_every,
+                                         T_rolling_window, optimize_every, rebalance_every, 
+                                         execution, cost,
                                          cpu_time_limit,
                                          return_portfolio, return_returns)
   }
@@ -300,7 +332,8 @@ benchmarkBacktest <- function(dataset_list, benchmark, price_name,
 singlePortfolioBacktest <- function(portfolio_fun, dataset_list, price_name, market,
                                     paral_datasets, show_progress_bar,
                                     shortselling, leverage,
-                                    T_rolling_window, optimize_every, rebalance_every,
+                                    T_rolling_window, optimize_every, rebalance_every, 
+                                    execution, cost,
                                     cpu_time_limit,
                                     return_portfolio, return_returns) {
   # create the progress bar
@@ -318,7 +351,8 @@ singlePortfolioBacktest <- function(portfolio_fun, dataset_list, price_name, mar
     for (i in 1:length(dataset_list)) {
       result[[i]] <- singlePortfolioSingleXTSBacktest(portfolio_fun, dataset_list[[i]], price_name, market,
                                                       shortselling, leverage,
-                                                      T_rolling_window, optimize_every, rebalance_every,
+                                                      T_rolling_window, optimize_every, rebalance_every, 
+                                                      execution, cost,
                                                       cpu_time_limit,
                                                       return_portfolio, return_returns)
       if (show_progress_bar) opts$progress(i) # show progress bar
@@ -332,7 +366,8 @@ singlePortfolioBacktest <- function(portfolio_fun, dataset_list, price_name, mar
     result <- foreach(dat = dataset_list, .combine = c, .packages = .packages(), .export = ls(envir = .GlobalEnv), .options.snow = opts) %dopar% {
       return(list(singlePortfolioSingleXTSBacktest(portfolio_fun, dat, price_name, market,
                                                    shortselling, leverage,
-                                                   T_rolling_window, optimize_every, rebalance_every,
+                                                   T_rolling_window, optimize_every, rebalance_every, 
+                                                   execution, cost,
                                                    cpu_time_limit,
                                                    return_portfolio, return_returns)))
     }
@@ -344,6 +379,9 @@ singlePortfolioBacktest <- function(portfolio_fun, dataset_list, price_name, mar
   else
     names(result) <- names(dataset_list)
   
+  # in case portfolio_fun had params as attribute (for parameter tuning), keep it
+  attr(result, "params") <- attr(portfolio_fun, "param")
+
   return(result)
 }
 
@@ -354,7 +392,8 @@ singlePortfolioBacktest <- function(portfolio_fun, dataset_list, price_name, mar
 #' @import xts
 singlePortfolioSingleXTSBacktest <- function(portfolio_fun, data, price_name, market,
                                              shortselling, leverage,
-                                             T_rolling_window, optimize_every, rebalance_every,
+                                             T_rolling_window, optimize_every, rebalance_every, 
+                                             execution, cost,
                                              cpu_time_limit,
                                              return_portfolio, return_returns) {
   # create return container
@@ -362,18 +401,19 @@ singlePortfolioSingleXTSBacktest <- function(portfolio_fun, data, price_name, ma
               cpu_time = NA, 
               error = FALSE, 
               error_message = NA,
-              portfolio = NA,
+              w_designed = NA,
+              w_bop = NA,
               return = NA,
-              cumPnL = NA)
+              wealth = NA)
   
   # when portfolio_return is given
   if (market) {
     idx_prices_window <- data$index[-(1:(T_rolling_window-1))]
     idx_return <- diff(log(idx_prices_window), na.pad = FALSE)
     res$performance <- portfolioPerformance(rets = idx_return)
-    res$performance['ROT bps'] <- Inf
+    res$performance["ROT (bps)"] <- Inf
     res$cpu_time <- 0
-    if (return_returns) {res$return <- idx_return; res$cumPnL <- idx_prices_window[-1]/as.numeric(idx_prices_window[1])}
+    if (return_returns) {res$return <- idx_return; res$wealth <- idx_prices_window[-1]/as.numeric(idx_prices_window[1])}
     return(res)
   }
   
@@ -452,16 +492,25 @@ singlePortfolioSingleXTSBacktest <- function(portfolio_fun, data, price_name, ma
   
   # compute returns of portfolio
   R_lin <- PerformanceAnalytics::CalculateReturns(prices)
-  ret_port <- returnPortfolio(R = R_lin, weights = w)
-  rets <- ret_port$rets
+  portf <- returnPortfolio(R = R_lin, weights = w, execution = execution, cost = cost)
+  # sanity check:
+  # res_check <- PerformanceAnalytics::Return.portfolio(R_lin, weights = w, verbose = TRUE)
+  # all.equal(portf$rets, res_check$returns)
+  # all.equal(portf$w_bop, res_check$BOP.Weight, check.attributes = FALSE)
 
-  res$performance <- portfolioPerformance(rets = rets, ROT_bips = ret_port$ROT_bips)
+  res$performance <- portfolioPerformance(rets = portf$rets, ROT_bips = portf$ROT_bips)
   res$cpu_time <- mean(cpu_time)
   res$error <- error
   res$error_message <- error_message
-  if (return_portfolio) res$portfolio <- w
-  if (return_returns) {res$return <- rets; res$cumPnL <- cumprod(1 + rets)}
-  
+  if (return_portfolio) {
+    res$w_designed <- w
+    res$w_bop <- portf$w_bop
+  }
+  if (return_returns) {
+    res$return <- portf$rets
+    res$wealth <- portf$wealth
+  }
+
   return(res)
 }
 
@@ -474,7 +523,7 @@ singlePortfolioSingleXTSBacktest <- function(portfolio_fun, data, price_name, ma
 portfolioPerformance <- function(rets = NA, ROT_bips = NA) {
   performance <- rep(NA, 7)
   names(performance) <- c("Sharpe ratio", "max drawdown", "annual return", "annual volatility", 
-                          "Sterling ratio", "Omega ratio", "ROT bps")
+                          "Sterling ratio", "Omega ratio", "ROT (bps)")
   # "judge" means how to judge the performance, 1: the bigger the better, -1: the smaller the better
   attr(performance, "judge") <- c(1, -1, 1, -1, 1, 1, 1)
   
@@ -488,8 +537,7 @@ portfolioPerformance <- function(rets = NA, ROT_bips = NA) {
   performance["annual volatility"] <- PerformanceAnalytics::StdDev.annualized(rets)
   performance["Sterling ratio"]    <- PerformanceAnalytics::Return.annualized(rets) / PerformanceAnalytics::maxDrawdown(rets)
   performance["Omega ratio"]       <- PerformanceAnalytics::Omega(rets)
-  performance["ROT bps"]           <- ROT_bips
-  
+  performance["ROT (bps)"]         <- ROT_bips
   
   return(performance)
 }
@@ -501,10 +549,14 @@ portfolioPerformance <- function(rets = NA, ROT_bips = NA) {
 #   R: is an xts with the individual asset linear returns (not log returns)
 #   weights: is an xts with the normalized dollar allocation (wrt NAV, typically with sum=1) where
 #            - each row represents a rebalancing date (with portfolio computed with info up to and including that day)
-#            - dates with no rows means no rebalancing (note that the portfolio may then violate some margin constraints...)
+#            - dates with no rows means: no rebalancing (note that the portfolio may then violate some margin constraints...)
+#   rets: are the returns for each day at the close
 #
 #' @import xts
-returnPortfolio <- function(R, weights, execution = c("same day", "next day"), name = "portfolio.returns") {
+returnPortfolio <- function(R, weights, 
+                            execution = c("same day", "next day"), 
+                            cost = list(buy = 0*10^(-4), sell = 0*10^(-4), short = 0*10^(-4), long_leverage = 0*10^(-4)),
+                            initial_cash = 1) {
   ######## error control  #########
   if (!is.xts(R) || !is.xts(weights)) stop("This function only accepts xts")
   if (attr(index(R), "class") != "Date") stop("This function only accepts daily data")
@@ -513,50 +565,75 @@ returnPortfolio <- function(R, weights, execution = c("same day", "next day"), n
   if (anyNA(R[-1])) stop("Returns contain NAs")
   #################################
   
+  # transaction cost
+  compute_tc <- !all(cost == 0)
+  tc <- 0
+
   # fill in w with NA to match the dates of R and lag appropriately
   w <- R; w[] <- NA
   w[index(weights), ] <- weights
-  switch(match.arg(execution),  # w[t] is the portfolio held before the realization of price[t] and R[t]
-         "same day" = { w <- lag.xts(w) },
-         "next day" = { w <- lag.xts(w, 2) },
-         stop("Execution method unknown")
-  )
-  rebalance_indices <- which(!is.na(w[, 1]))
-  # loop    (NAV contains the NAV at the beginning of the day, like w,
+  w <- switch(match.arg(execution),  # w[t] used info up to (including) price[t]
+              "same day" = lag.xts(w, 1),  # w[t] is (idealistically) executed at price[t], so will multiply return[t+1]
+              "next day" = lag.xts(w, 2),  # w[t] is executed one period later at price[t+1], so will multiply return[t+2]
+              stop("Execution method unknown"))
+  after_rebalance_indices <- which(!is.na(w[, 1]))
+  # loop    (NAV contains the NAV at the beginning of the day before the open (i.e., end of previous day), like w,
   #          whereas ret contains the returns at the end of the day, so lag(ret) equals (NAV - lag(NAV))/lag(NAV))
   NAV <- ret <- xts(rep(NA, nrow(R)), order.by = index(R))
-  colnames(ret) <- name
+  colnames(ret) <- "portfolio return"
   colnames(NAV) <- "NAV"
   delta_rel <- xts(matrix(0, nrow(w), ncol(w)), order.by = index(w))
-  NAV[rebalance_indices[1]] <- 1  # initial NAV of 1$
-  w_eop <- w[rebalance_indices[1], ]  # I don't want to count the initial huge turnover
-  for (t in rebalance_indices[1]:nrow(R)) {
-    if (t > rebalance_indices[1])
-      NAV[t] <- NAV[t-1]*NAV_relchange  # just to keep track
-    if (t %in% rebalance_indices) {
+  NAV[after_rebalance_indices[1]] <- initial_cash
+  w_eop <- w[after_rebalance_indices[1], ]  # don't want to count the initial huge turnover
+  for (t in after_rebalance_indices[1]:nrow(R)) {
+    if (t > after_rebalance_indices[1])
+      NAV[t] <- NAV[t-1]*NAV_relchange
+    if (t %in% after_rebalance_indices) {
       delta_rel[t, ] <- w[t, ] - w_eop
-      cash <- 1 - sum(w[t, ])       # normalized cash wrt NAV
-      ret[t] <- sum(R[t, ]*w[t, ])  # recall w is normalized wrt NAV
+      if (compute_tc)
+        tc <- cost$buy*sum(pos(delta_rel[t, ])) + cost$sell*sum(pos(-delta_rel[t, ])) +  # trading cost
+              cost$long_leverage*max(0, sum(pos(w[t, ])) - 1) + cost$short*sum(pos(-w[t, ]))  # borrowing cost
+      cash <- 1 - sum(w[t, ]) - tc  # normalized cash wrt NAV[t]
+      ret[t] <- sum(R[t, ]*w[t, ]) - tc  # recall w is normalized wrt NAV[t]
       w_eop <- (1 + R[t, ])*w[t, ]  # new w but it is still normalized wrt previous NAV which is not the correct normalization
-    }
-    else {
-      w[t, ] <- w_eop  # just to keep track
-      cash <- 1 - sum(w_eop)       # normalized cash wrt NAV
-      ret[t] <- sum(R[t, ]*w_eop)  # recall w is normalized wrt NAV
-      w_eop <- (1 + R[t, ])*w_eop  # new w but it is still normalized wrt previous NAV which is not the correct normalization
+    } else {
+      w[t, ] <- w_eop  # and delta_rel[t, ] is (already) zero
+      if (compute_tc)
+        tc <- cost$long_leverage*max(0, sum(pos(w_eop)) - 1) + cost$short*sum(pos(-w_eop))  # borrowing cost (no trading cost here)
+      cash <- 1 - sum(w_eop) - tc  # normalized cash wrt NAV[t]
+      ret[t] <- sum(R[t, ]*w_eop) - tc
+      w_eop <- (1 + R[t, ])*w_eop
     }
     NAV_relchange <- cash + sum(w_eop)       # NAV_relchange(t+1) = NAV(t+1)/NAV(t)
+    if (NAV_relchange <= 0) {  # if bankruptcy
+      NAV[(t+1):nrow(R)] <- as.numeric(NAV[t]*NAV_relchange)
+      ret[(t+1):nrow(R)] <- 0
+      break
+    }
     w_eop <- as.vector(w_eop/NAV_relchange)  # now w_eop is normalized wrt the current NAV(t+1)
   }
-  
+
+  # prepare time series to return
+  rets <- ret[after_rebalance_indices[1]:nrow(ret), ]
+  wealth <- c(xts(1, index(w)[after_rebalance_indices[1] - 1]), cumprod(1 + rets))
+  colnames(wealth) <- "portfolio wealth"
+  # sanity check: all.equal(na.omit(lag(wealth)), na.omit(NAV), check.attributes = FALSE)
+  # sanity check: PnL <- diff(NAV); PnL_rel <- PnL/lag.xts(NAV); all.equal(na.omit(lag(ret)), na.omit(PnL_rel), check.attributes = FALSE)
+
   # compute ROT based on normalized dollars
-  PnL <- diff(NAV); colnames(PnL) <- "PnL"
-  PnL_rel <- PnL/lag.xts(NAV)
-  turnover_rel <- xts(rowSums(abs(delta_rel)), index(delta_rel))
-  sum_PnL_rel <- sum(PnL_rel, na.rm = TRUE)
-  sum_turnover_rel <- sum(turnover_rel, na.rm = TRUE) * length(rebalance_indices)/(length(rebalance_indices)-1)  # to compensate for the removed fist turnover
+  sum_turnover_rel <- sum(abs(delta_rel))  # turnover only for indices: after_rebalance_indices[-1] (only when rebalancing except the first)
+  sum_PnL_rel <- sum(ret[after_rebalance_indices[1]:(tail(after_rebalance_indices, 1)-1), ])  # returns only for indices: after_rebalance_indices[1] to after_rebalance_indices[end]-1
   ROT_bips <- 1e4*sum_PnL_rel/sum_turnover_rel
-  
-  return(list(rets = ret[rebalance_indices[1]:nrow(ret), ],
-              ROT_bips = ROT_bips))
-}
\ No newline at end of file
+
+  return(list(rets = rets,
+              wealth = wealth,
+              ROT_bips = ROT_bips,
+              w_bop = w[!is.na(w[, 1])]))
+}
+
+pos <- function(x)
+  pmax(0, as.numeric(x))
+
+#cost <- list(buy = 17*10^(-4), sell = 17*10^(-4), long = 1*10^(-4), short = 1*10^(-4))  # for HK
+#cost <- list(buy = 1*10^(-4), sell = 1*10^(-4), long = 1*10^(-4), short = 1*10^(-4))  # for US
+#cost <- list(buy = 15*10^(-4), sell = 15*10^(-4), long = 15*10^(-4), short = 15*10^(-4))  # for CH
