@@ -1,4 +1,4 @@
-#' @title Download stock data from the Internet.
+#' @title Download stock data from the Internet
 #'
 #' @description This function is basically a robust wrapper for 
 #' \code{\link[quantmod:getSymbols]{quantmod:getSymbols}} to download stock 
@@ -17,6 +17,10 @@
 #' @param to String as the ending date (not included), e.g., "2017-09-17".
 #' @param rm_stocks_with_na Logical value indicating whether to remove stocks with missing values 
 #'                          (ignoring leading missing values). Default is \code{TRUE}.
+#' @param local_file_path Path where the stock data will be saved after the first time is downloaded, 
+#'                        so that in future retrievals it will be locally loaded (if the same 
+#'                        arguments are used). Default is \code{getwd()}. If local caching is not 
+#'                        desired, it can be deactivated by setting \code{local_file_path = NULL}.
 #' @param ... Additional arguments to be passed to \code{\link[quantmod:getSymbols]{quantmod:getSymbols}}.
 #'
 #' @return List of 7 \code{xts} objects named `open`, `high`, `low`, `close`, `volume`, 
@@ -38,8 +42,27 @@
 #' 
 #' @import xts 
 #'         quantmod
+#'         digest
 #' @export
-stockDataDownload <- function(stock_symbols, index_symbol = NULL, from, to, rm_stocks_with_na = TRUE, ...) {
+stockDataDownload <- function(stock_symbols, index_symbol = NULL, from, to, rm_stocks_with_na = TRUE, local_file_path = getwd(), ...) {
+  # some error control
+  if (missing(from) || missing(to)) stop("Arguments from and to have to be passed.")
+  
+  # first check if data locally saved
+  if (!is.null(local_file_path)) {
+    sorted_stock_symbols <- sort(stock_symbols)
+    attributes(sorted_stock_symbols) <- attributes(stock_symbols)
+    hash_stock_symbols <- digest(stock_symbols)
+    filename <- file.path(local_file_path,
+                          paste0("stockdata", "_from_", from, "_to_", to, "_(", hash_stock_symbols, ").RData"))
+    if (file.exists(filename)) {
+      message("Loading stock data from local file ", filename)
+      load(filename)
+      return(stockdata)
+    }
+  }
+  
+  # if not continue to download data
   open <- high <- low <- close <- volume <- adjusted <- list()
   n_stocks <- length(stock_symbols)
   message("Downloading ", n_stocks, " stocks...")
@@ -72,33 +95,49 @@ stockDataDownload <- function(stock_symbols, index_symbol = NULL, from, to, rm_s
   if (valid_count == 0) stop("Failed to download data from any stock.", call. = FALSE)
   if (valid_count < n_stocks) message("Failed to download: ", stocks_fail, "\n")
   
-  rt <- list("open"     = multipleXTSMerge(open),
-             "high"     = multipleXTSMerge(high),
-             "low"      = multipleXTSMerge(low),
-             "close"    = multipleXTSMerge(close),
-             "volume"   = multipleXTSMerge(volume),
-             "adjusted" = multipleXTSMerge(adjusted))
+  stockdata <- list("open"     = multipleXTSMerge(open),
+                    "high"     = multipleXTSMerge(high),
+                    "low"      = multipleXTSMerge(low),
+                    "close"    = multipleXTSMerge(close),
+                    "volume"   = multipleXTSMerge(volume),
+                    "adjusted" = multipleXTSMerge(adjusted))
   
   # if required, remove stocks with non-leading missing data
   if (rm_stocks_with_na) {
-    na_nonleading_mask <- apply(rt$adjusted, 2, function(x) {any(diff(is.na(x)) > 0)})
-    rt <- lapply(rt, function(x) {x[, !na_nonleading_mask]})
+    na_nonleading_mask <- apply(stockdata$adjusted, 2, function(x) {any(diff(is.na(x)) > 0)})
+    stockdata <- lapply(stockdata, function(x) {x[, !na_nonleading_mask]})
   }
   
+  # sanity check
+  ncols <- sapply(stockdata, ncol)
+  if (any(ncols[1] != ncols[-1])) stop("Number of cols of Op, Hi, Lo, Vo, Ad does not coincide!")
+      
+  
   # also download index data
   if (is.null(index_symbol)) index_symbol <- attr(stock_symbols, "index_symbol")  
   if (!is.null(index_symbol)) {
     message("Downloading index ", index_symbol, "...\n")
-    rt$index <- tryCatch(Ad(suppressWarnings(getSymbols(index_symbol, from = from, to = to, auto.assign = FALSE, ...))),
+    stockdata$index <- tryCatch(Ad(suppressWarnings(getSymbols(index_symbol, from = from, to = to, auto.assign = FALSE, ...))),
                          error = function(e) {stop("Failed to download index \"", index_symbol, "\"\n", call. = FALSE)})
     # check if the date of stock prices and market index match
-    if (any(index(rt$adjusted) != index(rt$index)))
+    if (any(index(stockdata$adjusted) != index(stockdata$index)))
       stop("Date of stocks prices and market index do not match.", call. = FALSE)
   }
   
-  return(rt)
+  # sanity check
+  nrows <- sapply(stockdata, nrow)
+  if (any(nrows[1] != nrows[-1])) stop("Number of rows of Op, Hi, Lo, Vo, Ad (or Index) does not coincide!")
+  
+  # save to local file if necessary for future usage
+  if (!is.null(local_file_path)) {
+    message("Saving stock data to local file ", filename, " for future use.")
+    save(stockdata, file = filename)
+  }
+
+  return(stockdata)
 }
 
+
 multipleXTSMerge <- function(xts_list) {
   res <- xts_list[[1]]
   if (length(xts_list) == 1) return(res)
@@ -109,7 +148,7 @@ multipleXTSMerge <- function(xts_list) {
 
 
 
-#' @title Generate random resamples from stock data.
+#' @title Generate random resamples from stock data
 #' 
 #' @description This function resamples the stock data downloaded by
 #' \code{\link{stockDataDownload}} to obtain many datasets for a 
@@ -122,8 +161,8 @@ multipleXTSMerge <- function(xts_list) {
 #' @param N_sample Number of stocks in each resample.
 #' @param T_sample Length of each resample (consecutive samples with a random initial time).
 #' @param num_datasets Number of resampled datasets (chosen randomly among the stock universe).
-#' @param check_stocks_with_na Logical value indicating whether to quit in case of stocks with missing 
-#'                             values (ignoring leading missing values). Default is \code{TRUE}.
+#' @param rm_stocks_with_na Logical value indicating whether to remove stocks with missing values 
+#'                          (ignoring leading missing values). Default is \code{TRUE}.
 #' 
 #' @return List of datasets resampled from \code{X}.
 #' 
@@ -146,33 +185,42 @@ multipleXTSMerge <- function(xts_list) {
 #' @import xts
 #'         zoo
 #' @export
-stockDataResample <- function(X, N_sample = 50, T_sample = 2*252, num_datasets = 10, check_stocks_with_na = TRUE) {
+stockDataResample <- function(X, N_sample = 50, T_sample = 2*252, num_datasets = 10, rm_stocks_with_na = TRUE) {
   # check data time zone
-  if (any(index(X$open) != index(X$index))) stop("The date indexes of \"X\" are not matched.")
+  if ((!is.null(X$index)) && any(index(X$open) != index(X$index))) stop("The date indexes of \"X\" do not match.")
   
   # if required, remove stocks with non-leading missing data
-  if (check_stocks_with_na) {
+  if (rm_stocks_with_na) {
     na_nonleading_mask <- apply(X$open, 2, function(x) {any(diff(is.na(x)) > 0)})
     if (any(na_nonleading_mask)) stop("\"X\" does not satisfy monotone missing-data pattern.")
   }
   
-  N <- ncol(X[[1]])
+  # resampling
+  cols <- sapply(X, ncol)
+  N <- max(cols)  # some elements will have N cols, others just 1
+  elems_N <- which(cols == N)
+  elems_1 <- if (any(cols == 1)) which(cols == 1)
+             else NULL
   T <- nrow(X[[1]])
   if (T < T_sample) stop("\"T_sample\" can not be greater than the date length of \"X\".")
-  dataset <- list()
+  dataset <- vector("list", num_datasets)
   for (i in 1:num_datasets) {
-    
     t_start <- sample(T-T_sample+1, 1)
     t_mask <- t_start:(t_start+T_sample-1)
-    
-    mask <- rep(1:N)[!is.na(X[[1]][t_start, ])]
-    if (length(mask) <= N_sample)
-      stock_mask <- mask
-    else
-      stock_mask <- sample(mask, N_sample)
-    dataset[[i]] <- lapply(X[1:6], function(x){x[t_mask, stock_mask]})
-    dataset[[i]]$index <- X$index[t_mask, ]
+    N_mask <- rep(1:N)[!is.na(X[[1]][t_start, ])]
+    stock_mask <- if (length(N_mask) <= N_sample) N_mask
+                  else sample(N_mask, N_sample)
+    dataset[[i]] <- vector("list", length(X))
+    names(dataset[[i]]) <- names(X)
+    dataset[[i]][elems_N] <- lapply(X[elems_N], function(x) {x[t_mask, stock_mask]})
+    if (!is.null(elems_1)) dataset[[i]][elems_1] <- lapply(X[elems_1], function(x) {x[t_mask, ]})
   }
   names(dataset) <- paste("dataset", 1:num_datasets)
+  message(sprintf("%d datasets resampled (with N = %d stocks and length T = %d) from the stock data between %s and %s.", 
+                  num_datasets, N_sample, T_sample, index(first(X$adjusted)), index(last(X$adjusted))))
+
   return(dataset)
 }
+
+
+
